#!/usr/bin/perl -w
#
# Copyright (c) 2007 VMware, Inc.  All rights reserved.
# Grab a single CPU counter (CPU Usage) for a Host System (all instances of CPU). 

use strict;
use warnings;

use FindBin;
use lib "$FindBin::Bin/../";

use VMware::VIRuntime;
use AppUtil::HostUtil;
use AppUtil::VMUtil;

$Util::script_version = "1.0";

sub retrieve_performance;

my %opts = (
   'hostname' => {
      type => "=s",
      help => "Name of the Host System",
      required => 1,
   },
);

Opts::add_options(%opts);
Opts::parse();


Util::connect();

my $perfmgr_view;
my $cpu_counters;

retrieve_performance();

Util::disconnect();

sub retrieve_performance() {
   my $host = Vim::find_entity_view(view_type => "HostSystem",
                                    filter => {'name' => Opts::get_option('hostname')});
   if (!defined($host)) {
      Util::trace(0,"HostSystem ".Opts::get_option('hostname')." not found.\n");
      return;
   }
   
   # get performance manager

   $perfmgr_view = Vim::get_view(mo_ref => Vim::get_service_content()->perfManager);
   my $perfCounterInfo = $perfmgr_view->perfCounter;

   foreach (@$perfCounterInfo) 
   {
      
         my $key = $_->key;
	     my $group_info = $_->groupInfo;
         my $name_info = $_->nameInfo;
         if ($group_info->key eq 'cpu' && ($name_info->key eq 'usage')) 
         {
            $cpu_counters->{ $key } = $_;
         } 
      
   }

   # get refresh rate of the entity (Host System)
   my $interval = get_perf_interval($host); 

   my @perf_metric_ids = filter_metric_ids($perfmgr_view->QueryAvailablePerfMetric(entity => $host,
	                                                                               intervalId => $interval));
   
   
   my $perf_query_spec = PerfQuerySpec->new(entity => $host,
                                         	metricId => @perf_metric_ids,
                                         	format => 'csv',
										    intervalId => $interval,
											maxSample => 1);

   # get performance data
   my $perf_data;
   eval {
      $perf_data = $perfmgr_view->QueryPerf(querySpec => $perf_query_spec);
   };
   if ($@) {
      if (ref($@) eq 'SoapFault') {
         if (ref($@->detail) eq 'InvalidArgument') {
            Util::trace(0,"Specified parameters are not correct. \n");
         }
      }
      return;
   }
   if (! @$perf_data) {
      Util::trace(0,"Either Performance data not available for requested period "
                    ."or instance is invalid \n");
      return;
   }
   print "\n";
   foreach (@$perf_data) {
      Util::trace(0,"Performance data for: " . $host->name . "\n");
	  print "\n";
      my $time_stamps = $_->sampleInfoCSV;
      my $values = $_->value;
      foreach (@$values) {
         print_counter_info($_->id->counterId, $_->id->instance);
         Util::trace(0,"Sample info : " . $time_stamps."\n");
         Util::trace(0,"Value: " . $_->value . "\n");
         print "\n";
      }
   }
}

sub filter_metric_ids 
{
   my ($perf_metric_ids) = @_;
   my @filtered_list;
   foreach (@$perf_metric_ids) 
   {
      if (exists $cpu_counters->{$_->counterId}) 
      {
         my $metric = PerfMetricId->new (counterId => $_->counterId,
                                          instance => "*");
         push @filtered_list, $metric;
      }
   }
   return \@filtered_list;
}

sub get_perf_interval
{
   my $entity = shift;
   my $provider_summary = $perfmgr_view->QueryPerfProviderSummary(entity => $entity);
   my $interval = $provider_summary->refreshRate;
   return $interval;
}


sub print_counter_info {
   my ($counter_id, $instance) = @_;
   my $counter = $cpu_counters->{$counter_id};
   if (defined $counter) {
   Util::trace(0,"Counter: " . $counter->nameInfo->label."\n");
   if (defined $instance) {
      Util::trace(0,"Instance : " . $instance."\n");
   }
      Util::trace(0,"Description: " . $counter->nameInfo->summary."\n");
      Util::trace(0,"Units: " . $counter->unitInfo->label."\n");
   }
}

__END__

=head1 NAME

hostCpuCounters.pl - Retrieves performance counter cpu usage from all cpu instances of a vm

=head1 SYNOPSIS

 hostCpuCounters.pl [options]

=head1 DESCRIPTION

This VI Perl command-line utility provides an interface to retrieve
cpu usagemhz performance counters from the specified vm. 

=head1 OPTIONS

=head2 GENERAL OPTIONS

=over

=item B<hostname>

Required. Name of the Host System.

=back

=head1 EXAMPLES

Retrieve performance counter 'usage' for countertype 'cpu' from Host System '192.168.0.0'

 hostCpuCounters.pl --url https://<host>:<port>/sdk/vimService
                --username myuser --password mypassword
                --hostname 192.168.0.0 

=head1 SUPPORTED PLATFORMS

All operations work with VMware VirtualCenter 2.0.1 or later.

All operations work with VMware ESX Server 3.0.1 or later.
