#!/usr/bin/perl -w
#
# Copyright (c) 2007 VMware, Inc.  All rights reserved.
# Ex - 4 Grab a set of 4 memory counters (active, swapin, swapout, ballooned/memctl) for a VM. 

use strict;
use warnings;

use FindBin;
use lib "$FindBin::Bin/../";

use VMware::VIRuntime;
use AppUtil::HostUtil;
use AppUtil::VMUtil;

$Util::script_version = "1.0";

sub retrieve_performance;

my %opts = (
   'vmname' => {
      type => "=s",
      help => "Name of the vm",
      required => 1,
   },
);

Opts::add_options(%opts);
Opts::parse();


Util::connect();

my $perfmgr_view;
my $mem_counters;
my $all_counters;

retrieve_performance();

Util::disconnect();

sub retrieve_performance() {
   my $vm = Vim::find_entity_view(view_type => "VirtualMachine",
                                    filter => {'name' => Opts::get_option('vmname')});
   if (!defined($vm)) {
      Util::trace(0,"VirtualMachine ".Opts::get_option('vm')." not found.\n");
      return;
   }
   
   $perfmgr_view = Vim::get_view(mo_ref => Vim::get_service_content()->perfManager);
   my $perfCounterInfo = $perfmgr_view->perfCounter;
   foreach (@$perfCounterInfo) 
	{
      
	  if($_->rollupType->val =~/average/)
		{
      	 my $key = $_->key;
	     my $group_info = $_->groupInfo;
         my $name_info = $_->nameInfo;
          if ($group_info->key eq 'mem') 
			{
			  if (($name_info->key eq 'active')||($name_info->key eq 'swapin')||($name_info->key eq 'swapout')||($name_info->key eq 'vmmemctl'))
			   {
         	     $mem_counters->{ $key } = $_;
			   }
      		} 
		}
	}
      my $interval = get_perf_interval($vm); 
	  my @perf_metric_ids = filter_metric_ids($perfmgr_view->QueryAvailablePerfMetric(entity => $vm, intervalId => $interval));
	  
	  
	  my $perf_query_spec = PerfQuerySpec->new(entity => $vm,
                                         	metricId => @perf_metric_ids,
                                         	format => 'csv',
										    intervalId => $interval,
											maxSample => 1);

	# get performance data
	my $perf_data;

    eval {
       $perf_data = $perfmgr_view->QueryPerf( querySpec => $perf_query_spec);
    };
    if ($@) {
       if (ref($@) eq 'SoapFault') {
         if (ref($@->detail) eq 'InvalidArgument') {
            Util::trace(0,"Specified parameters are not correct");
         }
      }
      return;
   }
   if (! @$perf_data) {
      Util::trace(0,"Either Performance data not available for requested period "
                    ."or instance is invalid\n");
      return;
   }
   foreach (@$perf_data) {
      Util::trace(0,"Performance data for: " . $vm->name . "\n");
      my $time_stamps = $_->sampleInfoCSV;
      my $values = $_->value;
      foreach (@$values) {
         print_counter_info($_->id->counterId, $_->id->instance);
         Util::trace(0,"Sample info : " . $time_stamps."\n");
         Util::trace(0,"Value: " . $_->value . "\n");
      }
   }
}

sub filter_metric_ids 
{
   my ($perf_metric_ids) = @_;
   my @filtered_list;
   foreach (@$perf_metric_ids) 
	{
      if (exists $mem_counters->{$_->counterId}) 
		{
		    my $metric = PerfMetricId->new (counterId => $_->counterId,
                                          instance => "*");
            push @filtered_list, $metric;
         }
   }
   return \@filtered_list;
}

sub get_perf_interval
{
   my $entity = shift;
   my $provider_summary = $perfmgr_view->QueryPerfProviderSummary(entity => $entity);
   my $interval = $provider_summary->refreshRate;
   return $interval;
}


sub print_counter_info {
   my ($counter_id, $instance) = @_;
   my $counter = $mem_counters->{$counter_id};
  Util::trace(0,"Counter: " . $counter->nameInfo->label."\n\n");
   if (defined $instance) {
      Util::trace(0,"Instance : " . $instance."\n");
   }
   Util::trace(0,"Description: " . $counter->nameInfo->summary."\n");
   Util::trace(0,"Units: " . $counter->unitInfo->label."\n");
}




__END__

=head1 NAME

memcounters.pl - Retrieve vm memory performance counters (active, swapin, swapout, ballooned/memctl)
=head1 SYNOPSIS

memcounters.pl [options]

=head1 DESCRIPTION

This VI Perl command-line utility provides an interface to retrieve
memory swapin, active, swapout, ballooned performance counters from the specified vm. 

=head1 OPTIONS

=head2 GENERAL OPTIONS

=over

=item B<vmname>

Required. Name of the virtual machine.

=back

=head1 EXAMPLES

Retrieve performance counters for vm 'Vm123'

 memcounters.pl --url https://<host>:<port>/sdk/vimService
                --username myuser --password mypassword
                --vmname Vm123 

=head1 SUPPORTED PLATFORMS

All operations work with VMware VirtualCenter 2.0.1 or later.

All operations work with VMware ESX Server 3.0.1 or later.
