#!/usr/bin/perl -w
#
# Copyright (c) 2007 VMware, Inc.  All rights reserved.
# Grab the network bandwidth counters, for each NIC for a host.

use strict;
use warnings;

use FindBin;
use lib "$FindBin::Bin/../";

use VMware::VIRuntime;
use AppUtil::HostUtil;
use AppUtil::VMUtil;

$Util::script_version = "1.0";

sub retrieve_performance;

my %opts = (
   'hostname' => {
      type => "=s",
      help => "Name of the host",
      required => 1,
   },
);

Opts::add_options(%opts);
Opts::parse();


Util::connect();

my $datastores;
my $disk_counters;
my $perfmgr_view;


retrieve_performance();

Util::disconnect();

sub retrieve_performance() {
   my $host = Vim::find_entity_view(view_type => "HostSystem",
                                    filter => {'name' => Opts::get_option('hostname')});

   if (!defined($host)) {
      Util::trace(0,"Host " . Opts::get_option('hostname') . " not found.\n");
      return;
   }

   # get performance manager
   $perfmgr_view = Vim::get_view(mo_ref => Vim::get_service_content()->perfManager);
   my $perfCounterInfo = $perfmgr_view->perfCounter;

   foreach (@$perfCounterInfo) 
   {
      my $key = $_->key;
      my $group_info = $_->groupInfo;
      my $name_info = $_->nameInfo;
      if ($group_info->key eq 'net' && ($name_info->key eq 'received') || ($name_info->key eq 'transmitted'))
      {
         $disk_counters->{ $key } = $_;
      }
   }

   # get refresh rate of the entity (host)
   my $interval = get_perf_interval($host); 

   #get available disk performance metric Ids
   my @perf_metric_ids = filter_metric_ids($perfmgr_view->QueryAvailablePerfMetric(entity => $host, intervalId => $interval));
   
   my $perf_query_spec = PerfQuerySpec->new(entity => $host,
                                          metricId => @perf_metric_ids,
                                          format => 'csv',
                                          intervalId => $interval,
                                          maxSample => 1);

   # get performance data
   my $perf_data;
   eval {
      $perf_data = $perfmgr_view->QueryPerf(querySpec => $perf_query_spec);
   };
   if ($@) {
      if (ref($@) eq 'SoapFault') {
         if (ref($@->detail) eq 'InvalidArgument') {
            Util::trace(0,"Specified parameters are not correct. \n");
         }
      }
      return;
   }
   if (! @$perf_data) {
      Util::trace(0,"Either Performance data not available for requested period "
                    ."or instance is invalid \n");
      return;
   }

   # Display performance stats per nic instance
   Util::trace(0,"Performance data for: " . $host->name . "\n");
   Util::trace(0,"-------------------------------------------------\n");

   foreach (@$perf_data) {
      my $time_stamps = $_->sampleInfoCSV;
      my $values = $_->value;
      foreach (@$values) {
         if ($_->id->instance) {
            print_counter_info($_->id->counterId, $_->id->instance);
            Util::trace(0,"Sample info : " . $time_stamps."\n");
            Util::trace(0,"Value: " . $_->value . "\n");
         }
      }
   }
}

sub filter_metric_ids 
{
   my ($perf_metric_ids) = @_;
   my @filtered_list;

   # Filter the counters based on net group
   foreach (@$perf_metric_ids) 
   {
      if (exists $disk_counters->{$_->counterId}) 
      {
         my $counterid = $_->counterId;
         my $metric = PerfMetricId->new(counterId => $_->counterId,
                                        instance => $_->instance);
         push @filtered_list, $metric;
      }
   }
   return \@filtered_list;
}


sub get_perf_interval
{
   my $entity = shift;
   my $provider_summary = $perfmgr_view->QueryPerfProviderSummary(entity => $entity);
   my $interval = $provider_summary->refreshRate;
   return $interval;
}


sub print_counter_info {
   my ($counter_id, $instance) = @_;
   my $counter = $disk_counters->{$counter_id};
   Util::trace(0,"\nCounter: " . $counter->nameInfo->label."\n");
   if (defined $instance) {
      Util::trace(0,"Instance : " . $instance."\n");
   }
   Util::trace(0,"Description: " . $counter->nameInfo->summary."\n");
   Util::trace(0,"Units: " . $counter->unitInfo->label."\n");
}

__END__

=head1 NAME

networkcounters.pl - Retrieves the network bandwidth counters, for each NIC for a host.

=head1 SYNOPSIS

networkcounters.pl [options]

=head1 DESCRIPTION

This VI Perl command-line utility provides an interface to retrieve
the network bandwidth counters, for each NIC for a host.

=head1 OPTIONS

=head2 GENERAL OPTIONS

=over

=item B<vmname>

Required. Name of the virtual machine.

=back

=head1 EXAMPLES

Retrieve disk performance counter for all datastores instances of a host 'hostname'

networkcounters.pl --url https://<host>:<port>/sdk/vimService
                --username myuser --password mypassword
                --hostname hostname

=head1 SUPPORTED PLATFORMS

All operations work with VMware VirtualCenter 2.0.1 or later.

All operations work with VMware ESX Server 3.0.1 or later.