#!/usr/bin/perl -w
#
# Copyright (c) 2007 VMware, Inc.  All rights reserved.
# Ex 3 - Grab a single CPU counter (CPU UsageMhz) for a VM (all instances of CPU). 

use strict;
use warnings;

use FindBin;
use lib "$FindBin::Bin/../";

use VMware::VIRuntime;
use AppUtil::HostUtil;
use AppUtil::VMUtil;

$Util::script_version = "1.0";

sub retrieve_performance;

my %opts = (
   'vmname' => {
      type => "=s",
      help => "Name of the vm",
      required => 1,
   },
);

Opts::add_options(%opts);
Opts::parse();


Util::connect();

my $perfmgr_view;
my $cpu_counters;


retrieve_performance();

Util::disconnect();

sub retrieve_performance() {
   my $vm = Vim::find_entity_view(view_type => "VirtualMachine",
                                    filter => {'name' => Opts::get_option('vmname')});
   if (!defined($vm)) {
      Util::trace(0,"VirtualMachine ".Opts::get_option('vm')." not found.\n");
      return;
   }
   
   # get performance manager

   $perfmgr_view = Vim::get_view(mo_ref => Vim::get_service_content()->perfManager);
   my $perfCounterInfo = $perfmgr_view->perfCounter;

   foreach (@$perfCounterInfo) 
   {
      if($_->rollupType->val =~/average/)
      {
         my $key = $_->key;
	     my $group_info = $_->groupInfo;
         my $name_info = $_->nameInfo;
         if ($group_info->key eq 'cpu' && ($name_info->key eq 'usagemhz')) 
         {
            $cpu_counters->{ $key } = $_;
         } 
      }
   }

   # get refresh rate of the entity (vm)
   my $interval = get_perf_interval($vm); 
   my @perf_metric_ids = filter_metric_ids($perfmgr_view->QueryAvailablePerfMetric(entity => $vm, intervalId => $interval));
   
  
   my $perf_query_spec = PerfQuerySpec->new(entity => $vm,
                                         	metricId => @perf_metric_ids,
                                         	format => 'csv',
										    intervalId => $interval,
											maxSample => 1);

   # get performance data
   my $perf_data;
   eval {
      $perf_data = $perfmgr_view->QueryPerf(querySpec => $perf_query_spec);
   };
   if ($@) {
      if (ref($@) eq 'SoapFault') {
         if (ref($@->detail) eq 'InvalidArgument') {
            Util::trace(0,"Specified parameters are not correct. \n");
         }
      }
      return;
   }
   if (! @$perf_data) {
      Util::trace(0,"Either Performance data not available for requested period "
                    ."or instance is invalid \n");
      return;
   }
   foreach (@$perf_data) {
      Util::trace(0,"Performance data for: " . $vm->name . "\n");
      my $time_stamps = $_->sampleInfoCSV;
      my $values = $_->value;
      foreach (@$values) {
         print_counter_info($_->id->counterId, $_->id->instance);
         Util::trace(0,"Sample info : " . $time_stamps."\n");
         Util::trace(0,"Value: " . $_->value . "\n");
        
      }
   }
}

sub filter_metric_ids 
{
   my ($perf_metric_ids) = @_;
   my @filtered_list;
   foreach (@$perf_metric_ids) 
   {
      if (exists $cpu_counters->{$_->counterId}) 
      {
         my $metric = PerfMetricId->new (counterId => $_->counterId,
                                          instance => "*");
         push @filtered_list, $metric;
      }
   }
   return \@filtered_list;
}

sub get_perf_interval
{
   my $entity = shift;
   my $provider_summary = $perfmgr_view->QueryPerfProviderSummary(entity => $entity);
   my $interval = $provider_summary->refreshRate;
   return $interval;
}


sub print_counter_info {
   my ($counter_id, $instance) = @_;
   my $counter = $cpu_counters->{$counter_id};
   Util::trace(0,"Counter: " . $counter->nameInfo->label."\n\n");
   if (defined $instance) {
      Util::trace(0,"Instance : " . $instance."\n");
   }
   Util::trace(0,"Description: " . $counter->nameInfo->summary."\n");
   Util::trace(0,"Units: " . $counter->unitInfo->label."\n");
}

__END__

=head1 NAME

vmcpuusageall.pl - Retrieves performance counter cpu usage from all cpu instances of a vm

=head1 SYNOPSIS

 vmcpuusageall.pl [options]

=head1 DESCRIPTION

This VI Perl command-line utility provides an interface to retrieve
cpu usagemhz performance counters from the specified vm. 

=head1 OPTIONS

=head2 GENERAL OPTIONS

=over

=item B<vmname>

Required. Name of the virtual machine.

=back

=head1 EXAMPLES

Retrieve performance counter usagemhz for countertype 'cpu' from vm 'Vm123'

 vmcpuusageall.pl --url https://<host>:<port>/sdk/vimService
                --username myuser --password mypassword
                --vmname Vm123 

=head1 SUPPORTED PLATFORMS

All operations work with VMware VirtualCenter 2.0.1 or later.

All operations work with VMware ESX Server 3.0.1 or later.
